/*RUN*/

// initial settings .....................................

s.options.numInputBusChannels = 2; // 4 for installation 2 for webstream
s.options.numOutputBusChannels = 2;

// create 8 control busses to pass data between synths
~ctrlBus = 8.collect({arg i; Bus.control(s,1); });

//~dataPath = "/home/rob/witw/data/"; // laptop
~dataPath = "/home/rizoma/witw/data/"; // installation computer
~devices = ["D7:80:BE:2C:63:BD", "F1:CC:C3:0B:7F:5C"];

// load and process the data source /////////////////////////////////
~csvFile=CSVFileReader.read(~dataPath ++ ~devices[0] ++ "/" ++ "sensor_data.csv",true,true);

// flatten ~data from 2D Array to regular array (per column);
~data = ( ~csvFile[0].size ).collect({ arg itt;
	Array.fill( ~csvFile.size-1,{ arg i;
		~csvFile.at(i+1).asArray.at(itt)}); // +1 to skip the column name
});

s.waitForBoot{

	// define synths

	SynthDef.new(\sine, {
		arg freq=440, modfreq=6, pmindex=3, modphase=0,
		panfreq=1.3, panphase=0.5, rel=0.5, gate=0, amp=0.0001;
		var sig, env;
		sig = PMOsc.ar(freq, modfreq, pmindex, modphase);
		//env = Env.perc(0.04,rel);
		env = Env.new([ 0, 1, 0.3, 0], [ 0.01, rel ,1], [1,-1]);
		sig = sig * EnvGen.kr(env, doneAction:2);
		sig = Pan2.ar(sig, FSinOsc.kr(panfreq, modphase, 0.5));
		//sig = PanAz.ar(4, sig, LFSaw.kr(panfreq, modphase), width: 1, orientation: 0);
		sig = sig * amp;
		Out.ar(0, sig);
	}).add;

	// initial settings .....................................

	~gr = Group.new;
	~pitchset = [20,40,60,80].midicps;

	// initial settings .....................................

	~dur = PatternProxy.new; // duration
	~deg = PatternProxy.new; // carrier frequency
	~mfr = PatternProxy.new; // modulation frequency
	~pmi = PatternProxy.new; // pulse modulation index
	~mph = PatternProxy.new; // modulation phase
	~pfr = PatternProxy.new; // pan frequency
	~oct = PatternProxy.new; // octave
	~amp = PatternProxy.new; // amplitude
	~sca = PatternProxy.new; // scale
	~rel = PatternProxy.new; // release

	~dur.source = Pseq(Array.geom(30, rrand(2,2.5), rrand(0.98,0.99)).mirror,inf);
	~deg.source = Prand([0,5,7],inf); // carrier frequency
	~mfr.source = Pexprand(0.1,3, inf); // modulation frequency
	~pmi.source = Pexprand(0.5,3, inf); // phase modulation index
	~mph.source = Pwhite(-1,1, inf); // modulation phase
	~pfr.source = Pexprand(0.1,1.5, inf); // pan freq

	// initial settings .....................................

	Pdef(\c, Pspawner({ | sp | 3.do { | i | sp.par( // 4 do for installation version
		Pbind( *[
			\instrument, \sine,
			\dur,      ~dur,
			\scale,    ~sca,
			\degree,   ~deg,
			\octave,   ~oct,
			\modfreq,  ~mfr,
			\pmindex,  ~pmi,
			\modphase, ~mph,
		    //\freq, exprand(~pitchset[0],~pitchset[0]*80).round(100),
			\panfreq,  ~pfr,
			\amp,      ~amp,
			\panphase, Pwhite(-1,1, inf),
			\rel,      ~rel,
			\group,    ~gr
		]);
	); sp.wait(10); }}); ).play;

	s.sync;

	( // normalize data ranges or not ///////////////////////////

		~temperature=~data[2];//.asFloat; //.normalize(0, 11);
		~dateTime=~data[0];
		~windDirection=~data[13];
		~windGust=~data[14]; // m/s
		~windNorth=~data[16]; // m/s
		~windEast=~data[18]; // m/s

		// data player ////////////////////////////////////////////

		~time = Routine {
			arg inval; var t = 0;
			// itterate through data at a fixed rate // data point every 15mins
			// 4sec = 1 hr 24 * 96seconds = 1 day // 9.6 seconds = 1 day @ 0.1.wait
			loop {
				t = (t + 1) % (~csvFile.size - 1);
				postln( t +
					" " + ~dateTime[t] +
					"C:" ++ ~temperature[t] +
					"Wind Dir: " + ~windDirection[t] +
					"Gust:" ++  ~windGust[t] +
					"North: " + ~windNorth[t] +
					"East: " + ~windEast[t]);
				0.1.wait;
		}}.play();

		// temporal structure of wind controled layer ///////////
		// ......................................................
		// conditional logic goes here //////////////////////////

		r = Routine { arg inval; var i = 0;

			loop {
				i = i + 1;  // this loop itterator is not used?
				postln(t);  // only t the ~time counter

				postf("beats: % seconds: % time: % \n",
					  thisThread.beats, thisThread.seconds, Main.elapsedTime );

				~sca.source = Scale.harmonicMinor;
				~deg.source = Prand([0,1,2,3,5,7],inf);
				~oct.source = Prand([4,5],inf);
				~dur.source = Pseq(Array.geom(30, rrand(2,4), rrand(0.96,0.97)).mirror,inf);
				~rel.source = Pexprand(2.1,5.5, inf);
				~amp.source = Pwhite(0.005, 0.01,inf);

				50.wait;
				postln("line 2");
				~deg.source = Prand([0,3,5,7],inf);
				~oct.source = Prand([3,4,5],inf);
				~dur.source = Pseq(Array.geom(30, rrand(1,2.5), rrand(0.96,0.97)).mirror,inf);

				50.wait;

				postln("line 3");
				~deg.source = Prand([0,3,5],inf);
				~oct.source = Prand([3,4,5],inf);
				~rel.source = Pexprand(1,3, inf);
				~dur.source = Pseq(Array.geom(30, rrand(1,1.5), rrand(0.96,0.97)).mirror,inf);

				50.wait;

				postln("line 4");
				~sca.source = Scale.chromatic;
				~deg.source = Prand([0,2,4,6,8,10],inf);
				~oct.source = Prand([3,4,5],inf);

				50.wait;

				postln("line 5");
				~deg.source = Prand([0,2,4,6,8,10],inf);
				~oct.source = Prand([3,4,5],inf);
				~dur.source = Pseq(Array.geom(30, rrand(0.5,0.6), rrand(0.98,0.99)).mirror,inf);
				~rel.source = Pexprand(0.5,3, inf);

				50.wait;

				postln("line 6");
				~deg.source = Prand([0,7,5],inf);
				~oct.source = Prand([2,3,4,5],inf);
				~dur.source = Pseq(Array.geom(30, rrand(1.5,3.6), rrand(0.98,0.99)).mirror,inf);
				~rel.source = Pexprand(2,4, inf);

				50.wait;

				postln("line 7");
				~sca.source = Scale.harmonicMinor;
				~deg.source = Prand([0,7,5],inf);
				~oct.source = Prand([2,3,4,5],inf);
				~dur.source = Pseq(Array.geom(30, rrand(1,1.5), rrand(0.98,0.99)).mirror,inf);
				~rel.source = Pexprand(2,6, inf);

				50.wait;


				postln("line 8");

				~deg.source = Prand([0,5,7,9],inf);
				~oct.source = Prand([4,5,6],inf);
				~dur.source = Pseq(Array.geom(30, rrand(2,3.5), rrand(0.98,0.99)).mirror,inf);
				~rel.source = Pexprand(6,12, inf);

				50.wait;

				postln("line 9");
				~deg.source = Prand([1,6,8,10],inf);
				~oct.source = Prand([3, 4,5],inf);
				~dur.source = Pseq(Array.geom(30, rrand(3,4.5), rrand(0.98,0.99)).mirror,inf);
				~rel.source = Pexprand(4,12, inf);

				50.wait;

			};
		}.play;
);

}; // closes s.waitForBoot .....................................

r.stop;
~time.stop;
